"""Sudo pair utilities."""
import grp
import os

from charmhelpers.core import hookenv, host, templating


def check_valid_group(group_name):
    """Check that a group exists."""
    try:
        grp.getgrnam(group_name)
        return True
    except KeyError:
        return False


def group_id(group_name):
    """Check that a group exists."""
    return grp.getgrnam(group_name).gr_gid


def group_names_to_group_ids(group_names):
    """
    Return comma-separated list of Group Ids.

    :param group_names: i.e. "root,user1,user2"
    :return gids: i.e. "0,1001,1002"
    """
    group_names = list(filter(check_valid_group, group_names.split(",")))
    return ",".join(map(str, (map(group_id, group_names))))


def copy_file(source, destination, owner, group, perms):
    """Copy a file on the unit."""
    if destination is not None:
        target_dir = os.path.dirname(destination)
        if not os.path.exists(target_dir):
            # This is a terrible default directory permission, as the file
            # or its siblings will often contain secrets.
            host.mkdir(os.path.dirname(destination), owner, group, perms=0o755)
        with open(source, "rb") as source_f:
            host.write_file(
                destination, source_f.read(), perms=perms, owner=owner, group=group
            )


class SudoPairHelper(object):
    """Configure sudo-pair."""

    def __init__(self):
        """Retrieve charm config and set defaults."""
        self.charm_config = hookenv.config()
        self.binary_path = "/usr/bin/sudo_approve"
        self.sudo_conf_path = "/etc/sudo.conf"
        self.sudoers_path = "/etc/sudoers"
        self.sudo_lib_path = "/usr/lib/sudo/sudo_pair.so"
        self.sudoers_bypass_path = "/etc/sudoers.d/91-bypass-sudopair-cmds"
        self.user_prompt_path = "/etc/sudo_pair.prompt.user"
        self.pair_prompt_path = "/etc/sudo_pair.prompt.pair"
        self.socket_dir = "/var/run/sudo_pair"
        self.tmpfiles_conf = "/usr/lib/tmpfiles.d/sudo_pair.conf"
        self.owner = "root"
        self.group = "root"
        self.socket_dir_perms = 0o644
        self.sudo_pair_so_perms = 0o644
        self.prompt_perms = 0o644
        self.sudoers_perms = 0o440
        self.sudo_conf_perms = 0o644
        self.sudo_approve_perms = 0o755

    def get_config(self):
        """Return config as a dict."""
        config = {
            "binary_path": self.binary_path,
            "user_prompt_path": self.user_prompt_path,
            "pair_prompt_path": self.pair_prompt_path,
            "socket_dir": self.socket_dir,
            "gids_enforced": group_names_to_group_ids(
                self.charm_config["groups_enforced"]
            ),
            "gids_exempted": group_names_to_group_ids(
                self.charm_config["groups_exempted"]
            ),
        }

        config.update(self.charm_config)
        return config

    def set_charm_config(self, charm_config):
        """Update configuration."""
        self.charm_config = charm_config

    def render_sudo_conf(self):
        """Render sudo.conf file."""
        return templating.render(
            "sudo.conf.tmpl",
            self.sudo_conf_path,
            self.get_config(),
            perms=self.sudo_conf_perms,
            owner=self.owner,
            group=self.group,
        )

    def create_socket_dir(self):
        """Create socket dir."""
        host.mkdir(
            self.socket_dir,
            perms=self.socket_dir_perms,
            owner=self.owner,
            group=self.group,
        )

    def create_tmpfiles_conf(self):
        """Create temporary conf file."""
        with open(self.tmpfiles_conf, "w") as f:
            f.write("d {} 0755 - - -\n".format(self.socket_dir))

    def install_sudo_pair_so(self):
        """Install sudo-pair lib."""
        sudo_pair_lib = os.path.join(hookenv.charm_dir(), "files", "sudo_pair.so")
        copy_file(
            sudo_pair_lib,
            self.sudo_lib_path,
            self.owner,
            self.group,
            self.sudo_pair_so_perms,
        )

    def copy_user_prompt(self):
        """Copy user prompt on the unit."""
        prompt_file = os.path.join(hookenv.charm_dir(), "files", "sudo.prompt.user")
        copy_file(
            prompt_file,
            self.user_prompt_path,
            self.owner,
            self.group,
            self.prompt_perms,
        )

    def copy_pair_prompt(self):
        """Copy pair prompt on the unit."""
        prompt_file = os.path.join(hookenv.charm_dir(), "files", "sudo.prompt.pair")
        copy_file(
            prompt_file,
            self.pair_prompt_path,
            self.owner,
            self.group,
            self.prompt_perms,
        )

    def copy_sudoers(self):
        """Copy sudoers file on the unit."""
        sudoers_file = os.path.join(hookenv.charm_dir(), "files", "sudoers")
        copy_file(
            sudoers_file, self.sudoers_path, self.owner, self.group, self.sudoers_perms
        )

    def render_sudo_approve(self):
        """Render sudo-approve file."""
        hookenv.log("Rendering sudo_approve.tmpl to {}".format(self.binary_path))
        return templating.render(
            "sudo_approve.tmpl",
            self.binary_path,
            self.get_config(),
            perms=self.sudo_approve_perms,
            owner=self.owner,
            group=self.group,
        )

    def render_bypass_cmds(self):
        """Render bypass command file."""
        if (
            self.get_config()["bypass_cmds"] != ""
            and self.get_config()["bypass_group"] != ""
        ):
            hookenv.log("Render bypass cmds to {}".format(self.sudoers_bypass_path))
            return templating.render(
                "91-bypass-sudopair-cmds.tmpl",
                self.sudoers_bypass_path,
                self.get_config(),
                perms=0o440,
                owner=self.owner,
                group=self.group,
            )
        return None

    def deconfigure(self):
        """Remove sudo-pair configuration."""
        paths = [
            self.sudo_conf_path,
            self.sudo_lib_path,
            self.binary_path,
            self.user_prompt_path,
            self.pair_prompt_path,
            self.sudoers_bypass_path,
            self.tmpfiles_conf,
        ]
        hookenv.log("Deleting: {}".format(paths))
        for path in paths:
            try:
                os.unlink(path)
            except Exception as e:
                # We're trying hard to delete all files, even if some might fail
                hookenv.log(
                    "Got exception unlinking {}: {}, continuing".format(path, e)
                )
