#!/usr/bin/python3.6
"""Charm functional tests."""

import os

import pytest

pytestmark = pytest.mark.asyncio

charm_build_dir = os.getenv("CHARM_BUILD_DIR").rstrip("/")


sources = [("local", "{}/sudo-pair".format(charm_build_dir))]

series = [
    "xenial",
    "bionic",
    "focal",
]


############
# FIXTURES #
############


@pytest.fixture(params=series)
def series(request):
    """Return ubuntu version (i.e. xenial) in use in the test."""
    return request.param


@pytest.fixture(params=sources, ids=[s[0] for s in sources])
def source(request):
    """Return source of the charm under test (i.e. local, cs)."""
    return request.param


@pytest.fixture
async def app(model, series, source):
    """Return application of the charm under test."""
    app_name = "sudo-pair-{}".format(series)
    return await model._wait_for_new("application", app_name)


@pytest.fixture
async def unit(app):
    """Return the sudo_pair unit we've deployed."""
    return app.units[0]


#########
# TESTS #
#########


async def test_deploy_app(model, series, source):
    """Deploy the sudo_pair app as a subordinate of ubuntu."""
    await model.deploy(
        "ubuntu", application_name="ubuntu-" + series, series=series, channel="stable"
    )
    sudo_pair_app = await model.deploy(
        source[1],
        application_name="sudo-pair-" + series,
        series=series,
        num_units=0,
        config={
            "bypass_cmds": "/bin/ls",
            "groups_enforced": "ubuntu",
            "bypass_group": "warthogs",
        },
    )
    await model.add_relation(
        "ubuntu-{}:juju-info".format(series), "sudo-pair-{}:juju-info".format(series)
    )

    await model.block_until(lambda: sudo_pair_app.status == "active")
    # no need to cleanup since the model will be be torn down at the end of the
    # testing


async def test_status(app):
    """Check that the app is in active state."""
    assert app.status == "active"


@pytest.mark.parametrize(
    "path,expected_stat",
    [
        ("/usr/lib/sudo/sudo_pair.so", {"gid": 0, "uid": 0, "mode": "0o100644"}),
        ("/usr/bin/sudo_approve", {"gid": 0, "uid": 0, "mode": "0o100755"}),
        ("/etc/sudo_pair.prompt.user", {"gid": 0, "uid": 0, "mode": "0o100644"}),
        ("/etc/sudo_pair.prompt.pair", {"gid": 0, "uid": 0, "mode": "0o100644"}),
        ("/var/run/sudo_pair", {"gid": 0, "uid": 0, "mode": "0o40644"}),
    ],
)
async def test_stats(path, expected_stat, unit, file_stat):
    """Check that created files have the correct permissions."""
    test_stat = await file_stat(path, unit)
    assert test_stat["size"] > 0
    assert test_stat["gid"] == expected_stat["gid"]
    assert test_stat["uid"] == expected_stat["uid"]
    assert test_stat["mode"] == expected_stat["mode"]


async def test_sudoers(file_contents, unit):
    """Check the content of sudoers file."""
    sudoers_content = await file_contents("/etc/sudoers", unit)
    assert "Defaults	log_output" in sudoers_content


async def test_sudoers_bypass_conf(file_contents, unit):
    """Check the content of sudoers bypass command file."""
    path = "/etc/sudoers.d/91-bypass-sudopair-cmds"
    sudoers_bypass_content = await file_contents(path=path, target=unit)
    content = "%warthogs ALL = (ALL) NOLOG_OUTPUT: /bin/ls"
    assert content in sudoers_bypass_content


async def test_reconfigure(reconfigure_app, file_contents, unit, app):
    """Change a charm config parameter and verify it is applied."""
    sudo_approve_path = "/usr/bin/sudo_approve"
    await reconfigure_app(cfg={"auto_approve": "false"}, target=app)
    sudo_approve_content = await file_contents(path=sudo_approve_path, target=unit)
    new_content = 'echo "You can\'t approve your own session."'
    assert new_content in sudo_approve_content


async def test_remove_relation(app, model, run_command):
    """Check that the relation is removed."""
    series = app.units[0].data["series"]
    app_name = "sudo-pair-{}".format(series)
    principalname = "ubuntu-{}".format(series)
    await app.remove_relation(
        "{}:juju-info".format(app_name), "{}:juju-info".format(principalname)
    )
    await model.block_until(lambda: not app.relations)
    principal = model.applications[principalname].units[0]
    res = await run_command("test -f /etc/sudo.conf || echo gone", target=principal)
    assert res["Stdout"].strip() == "gone"
    await model.add_relation(
        "{}:juju-info".format(principalname), "{}:juju-info".format(app_name)
    )
    await model.block_until(lambda: app.relations)
