"""Sudopair lib unit tests."""

import filecmp
import grp
import os

from libsudopair import check_valid_group, group_id


def test_check_valid_group():
    """Check an unix group is valid."""
    assert not check_valid_group("fake_group")
    assert check_valid_group(grp.getgrgid(os.getgid()).gr_name)


def test_group_id():
    """Verify group_id() is correct."""
    assert group_id(grp.getgrgid(os.getgid()).gr_name) == os.getgid()


class TestSudoPairHelper:
    """Module to test SudoPairHelper lib."""

    def test_pytest(self):
        """Assert testing is carryied using pytest."""
        assert True

    def test_sph(self, sph):
        """See if the sph fixture works to load charm configs."""
        assert isinstance(sph.charm_config, dict)

    def test_get_config(self, sph):
        """Check if config contains all the required entries."""
        default_keywords = [
            "binary_path",
            "user_prompt_path",
            "pair_prompt_path",
            "socket_dir",
            "gids_enforced",
            "gids_exempted",
        ]
        config = sph.get_config()
        for option in default_keywords:
            assert option in config

    def test_set_charm_config(self, sph):
        """Set new config."""
        charm_config = {
            "groups_enforced": "root",
            "groups_exempted": "",
            "bypass_cmds": "",
            "bypass_group": "",
            "auto_approve": True,
        }

        sph.set_charm_config(charm_config)

        for option in charm_config:
            assert option in sph.get_config()
            assert sph.get_config()[option] == charm_config[option]

    def test_render_sudo_conf(self, sph, tmpdir):
        """Check that sudo.conf is rendered correctly."""
        # Default config
        content = sph.render_sudo_conf()
        expected_content = (
            "Plugin sudo_pair sudo_pair.so binary_path={} "
            "user_prompt_path={} "
            "pair_prompt_path={} socket_dir={} gids_enforced={}".format(
                tmpdir.join("/usr/bin/sudo_approve"),
                tmpdir.join("/etc/sudo_pair.prompt.user"),
                tmpdir.join("/etc/sudo_pair.prompt.pair"),
                tmpdir.join("/var/run/sudo_pair"),
                "0",
            )
        )
        assert expected_content in content

        # Gid exempted
        groups_exempted = grp.getgrgid(os.getgid()).gr_name
        charm_config = {
            "groups_enforced": "root",
            "groups_exempted": groups_exempted,
            "bypass_cmds": "",
            "bypass_group": "",
            "auto_approve": True,
        }

        sph.set_charm_config(charm_config)
        expected_content = (
            "Plugin sudo_pair sudo_pair.so binary_path={} user_prompt_path={} "
            "pair_prompt_path={} socket_dir={} gids_enforced={} "
            "gids_exempted={}".format(
                tmpdir.join("/usr/bin/sudo_approve"),
                tmpdir.join("/etc/sudo_pair.prompt.user"),
                tmpdir.join("/etc/sudo_pair.prompt.pair"),
                tmpdir.join("/var/run/sudo_pair"),
                "0",
                os.getgid(),
            )
        )

        content = sph.render_sudo_conf()
        assert expected_content in content

        # Groups enforced
        groups_enforced = "root," + grp.getgrgid(os.getgid()).gr_name
        charm_config = {
            "groups_enforced": groups_enforced,
            "groups_exempted": "",
            "bypass_cmds": "",
            "bypass_group": "",
            "auto_approve": True,
        }
        sph.set_charm_config(charm_config)
        expected_content = (
            "Plugin sudo_pair sudo_pair.so binary_path={} user_prompt_path={} "
            "pair_prompt_path={} socket_dir={} gids_enforced={}".format(
                tmpdir.join("/usr/bin/sudo_approve"),
                tmpdir.join("/etc/sudo_pair.prompt.user"),
                tmpdir.join("/etc/sudo_pair.prompt.pair"),
                tmpdir.join("/var/run/sudo_pair"),
                "0,{}".format(os.getgid()),
            )
        )
        content = sph.render_sudo_conf()
        assert expected_content in content

    def test_render_bypass_cmds(self, sph):
        """Check that sudoers file is rendered correctly."""
        # Root bypass /bin/ls
        expected_content = "%root ALL = (ALL) NOLOG_OUTPUT: /bin/ls"
        charm_config = {
            "groups_enforced": "root",
            "groups_exempted": "",
            "bypass_cmds": "/bin/ls",
            "bypass_group": "root",
            "auto_approve": True,
        }
        sph.set_charm_config(charm_config)
        content = sph.render_bypass_cmds()
        assert expected_content in content

    def test_render_sudo_approve(self, sph, tmpdir):
        """Check that sudo_approve file is rendered correctly."""
        # Auto Approve true
        expected_content = "echo ${log_line} >> /var/log/sudo_pair.log"
        socket_dir = tmpdir.join("/var/run/sudo_pair")
        expected_content_socket_dir = 'declare -r SUDO_SOCKET_PATH="{}"'.format(
            socket_dir
        )
        content = sph.render_sudo_approve()
        assert expected_content in content
        assert expected_content_socket_dir in content

        # Auto Approve false
        expected_content = 'echo "You can\'t approve your own session."'
        charm_config = {
            "groups_enforced": "root",
            "groups_exempted": "",
            "bypass_cmds": "/bin/ls",
            "bypass_group": "root",
            "auto_approve": False,
        }
        sph.set_charm_config(charm_config)
        content = sph.render_sudo_approve()
        assert expected_content in content

    def test_create_socket_dir(self, sph, tmpdir):
        """Check that sudo_pair socket dir exists."""
        sph.create_socket_dir()
        assert os.path.exists(tmpdir.join("/var/run/sudo_pair"))

    def test_create_tmpfiles_conf(self, sph, tmpdir):
        """Check that sudo pair temporary conf is rendered correctly."""
        sph.create_tmpfiles_conf()
        expected_content = "d {} 0755 - - -\n".format(sph.socket_dir)
        with open(tmpdir.join("/usr/lib/tmpfiles.d/sudo_pair.conf")) as f:
            content = f.read()
        assert expected_content in content

    def test_install_sudo_pair_so(self, sph, tmpdir):
        """Check that sudo system lib exists."""
        sph.install_sudo_pair_so()
        assert filecmp.cmp(
            "./files/sudo_pair.so", tmpdir.join("/usr/lib/sudo/sudo_pair.so")
        )

    def test_copy_user_prompt(self, sph, tmpdir):
        """Check that user prompt exists."""
        sph.copy_user_prompt()
        assert filecmp.cmp(
            "./files/sudo.prompt.user", tmpdir.join("/etc/sudo_pair.prompt.user")
        )

    def test_copy_pair_prompt(self, sph, tmpdir):
        """Check that pair prompt exists."""
        sph.copy_pair_prompt()
        assert filecmp.cmp(
            "./files/sudo.prompt.pair", tmpdir.join("/etc/sudo_pair.prompt.pair")
        )

    def test_copy_sudoers(self, sph, tmpdir):
        """Check that sudoers file exists."""
        sph.copy_sudoers()
        assert filecmp.cmp("./files/sudoers", tmpdir.join("/etc/sudoers"))
